@@ -179,9 +179,9 @@ class TestPipelineBuilder:
     @patch('market_data_pipeline.pipeline_builder.SyntheticSource')
     @patch('market_data_pipeline.pipeline_builder.SecondBarAggregator')
     @patch('market_data_pipeline.pipeline_builder.HybridBatcher')
-    @patch('market_data_pipeline.pipeline_builder.StoreSink')
+    @patch('market_data_pipeline.pipeline_builder.create_store_sink')
     @patch('market_data_pipeline.pipeline_builder.StreamingPipeline')
-    def test_build_synthetic_bars_store(self, mock_pipeline, mock_store_sink, mock_batcher, 
+    def test_build_synthetic_bars_store(self, mock_pipeline, mock_create_store_sink, mock_batcher, 
                                        mock_operator, mock_source):
         """Test building a synthetic -> bars -> store pipeline."""
         config = Mock()
@@ -200,7 +200,8 @@ class TestPipelineBuilder:
         mock_bp = Mock()
         mock_bp.from_env_async = AsyncMock(return_value=mock_bp)
         mock_bp.from_env = Mock(return_value=mock_bp)
-        with patch('market_data_pipeline.pipeline_builder.AsyncBatchProcessor', mock_bp):
+        with patch('market_data_pipeline.pipeline_builder.AsyncBatchProcessor', mock_bp), \
+             patch.dict('os.environ', {'STORE_MODE': 'legacy'}):
             builder = PipelineBuilder(config=config)
             
             spec = PipelineSpec(
@@ -218,15 +219,15 @@ class TestPipelineBuilder:
             mock_source.assert_called_once()
             mock_operator.assert_called_once()
             mock_batcher.assert_called_once()
-            mock_store_sink.assert_called_once()
+            mock_create_store_sink.assert_called_once()
             mock_pipeline.assert_called_once()
 
     @patch('market_data_pipeline.pipeline_builder.ReplaySource')
     @patch('market_data_pipeline.pipeline_builder.SecondBarAggregator')
     @patch('market_data_pipeline.pipeline_builder.HybridBatcher')
-    @patch('market_data_pipeline.pipeline_builder.StoreSink')
+    @patch('market_data_pipeline.pipeline_builder.create_store_sink')
     @patch('market_data_pipeline.pipeline_builder.StreamingPipeline')
-    def test_build_replay_bars_store(self, mock_pipeline, mock_store_sink, mock_batcher,
+    def test_build_replay_bars_store(self, mock_pipeline, mock_create_store_sink, mock_batcher,
                                    mock_operator, mock_replay_source):
         """Test building a replay -> bars -> store pipeline."""
         config = Mock()
@@ -246,7 +247,11 @@ class TestPipelineBuilder:
         mock_bp = Mock()
         mock_bp.from_env_async = AsyncMock(return_value=mock_bp)
         mock_bp.from_env = Mock(return_value=mock_bp)
-        with patch('market_data_pipeline.pipeline_builder.AsyncBatchProcessor', mock_bp):
+        mock_store_sink = Mock()
+        mock_create_store_sink.return_value = mock_store_sink
+        
+        with patch('market_data_pipeline.pipeline_builder.AsyncBatchProcessor', mock_bp), \
+             patch.dict('os.environ', {'STORE_MODE': 'legacy'}):
             builder = PipelineBuilder(config=config)
             
             spec = PipelineSpec(
@@ -263,7 +268,7 @@ class TestPipelineBuilder:
             mock_replay_source.assert_called_once()
             mock_operator.assert_called_once()
             mock_batcher.assert_called_once()
-            mock_store_sink.assert_called_once()
+            mock_create_store_sink.assert_called_once()
             mock_pipeline.assert_called_once()
 
     def test_build_replay_source_not_available(self):
@@ -428,7 +433,8 @@ class TestPipelineBuilder:
             with pytest.raises(ConfigurationError, match="kafka sink requires kafka_bootstrap and kafka_topic"):
                 builder.build(spec)
 
-    def test_build_store_sink_amds_not_available(self):
+    @patch('market_data_pipeline.pipeline_builder.create_store_sink')
+    def test_build_store_sink_amds_not_available(self, mock_create_store_sink):
         """Test building store sink when AMDS not available."""
         config = Mock()
         config.batch_size = 500
@@ -440,7 +446,138 @@ class TestPipelineBuilder:
         config.sink_workers = 2
         config.drop_policy = "oldest"
         
-        with patch('market_data_pipeline.pipeline_builder.AsyncBatchProcessor', None):
+        # Mock the create_store_sink to raise the expected error
+        mock_create_store_sink.side_effect = ConfigurationError("market_data_store AsyncBatchProcessor not installed")
+        
+        with patch('market_data_pipeline.pipeline_builder.AsyncBatchProcessor', None), \
+             patch.dict('os.environ', {'STORE_MODE': 'legacy'}):
+            builder = PipelineBuilder(config=config)
+            
+            spec = PipelineSpec(
+                tenant_id="test_tenant",
+                pipeline_id="test_pipeline",
+                source="synthetic",
+                symbols=["AAPL"],
+                operator="bars",
+                sink="store"
+            )
+            
+            with pytest.raises(ConfigurationError, match="market_data_store AsyncBatchProcessor not installed"):
+                builder.build(spec)
+
+    # === Fixed tests for dual-sink system (Phase 20.1) ===
+    
+    @patch('market_data_pipeline.pipeline_builder.SyntheticSource')
+    @patch('market_data_pipeline.pipeline_builder.SecondBarAggregator')
+    @patch('market_data_pipeline.pipeline_builder.HybridBatcher')
+    @patch('market_data_pipeline.pipeline_builder.create_store_sink')
+    @patch('market_data_pipeline.pipeline_builder.StreamingPipeline')
+    def test_build_synthetic_bars_store_legacy(self, mock_pipeline, mock_create_store_sink, mock_batcher, 
+                                       mock_operator, mock_source):
+        """Test building a synthetic -> bars -> store pipeline with legacy sink."""
+        config = Mock()
+        config.batch_size = 500
+        config.flush_ms = 100
+        config.sink_queue_max = 100
+        config.pacing_max_per_sec = 1000
+        config.pacing_burst = 1000
+        config.ticks_per_sec = 100
+        config.bar_window_sec = 1
+        config.bar_allowed_lateness_sec = 0
+        config.sink_workers = 2
+        config.drop_policy = "oldest"
+        
+        # Mock AsyncBatchProcessor
+        mock_bp = Mock()
+        mock_bp.from_env_async = AsyncMock(return_value=mock_bp)
+        mock_bp.from_env = Mock(return_value=mock_bp)
+        mock_store_sink = Mock()
+        mock_create_store_sink.return_value = mock_store_sink
+        
+        with patch('market_data_pipeline.pipeline_builder.AsyncBatchProcessor', mock_bp), \
+             patch.dict('os.environ', {'STORE_MODE': 'legacy'}):
+            builder = PipelineBuilder(config=config)
+            
+            spec = PipelineSpec(
+                tenant_id="test_tenant",
+                pipeline_id="test_pipeline",
+                source="synthetic",
+                symbols=["AAPL", "MSFT"],
+                operator="bars",
+                sink="store"
+            )
+            
+            pipeline = builder.build(spec)
+            
+            # Verify components were created
+            mock_source.assert_called_once()
+            mock_operator.assert_called_once()
+            mock_batcher.assert_called_once()
+            mock_create_store_sink.assert_called_once()
+            mock_pipeline.assert_called_once()
+
+    @patch('market_data_pipeline.pipeline_builder.SyntheticSource')
+    @patch('market_data_pipeline.pipeline_builder.SecondBarAggregator')
+    @patch('market_data_pipeline.pipeline_builder.HybridBatcher')
+    @patch('market_data_pipeline.pipeline_builder.create_store_sink')
+    @patch('market_data_pipeline.pipeline_builder.StreamingPipeline')
+    def test_build_synthetic_bars_store_provider(self, mock_pipeline, mock_create_store_sink, mock_batcher, 
+                                       mock_operator, mock_source):
+        """Test building a synthetic -> bars -> store pipeline with provider sink."""
+        config = Mock()
+        config.batch_size = 500
+        config.flush_ms = 100
+        config.sink_queue_max = 100
+        config.pacing_max_per_sec = 1000
+        config.pacing_burst = 1000
+        config.ticks_per_sec = 100
+        config.bar_window_sec = 1
+        config.bar_allowed_lateness_sec = 0
+        config.sink_workers = 2
+        config.drop_policy = "oldest"
+        
+        mock_store_sink = Mock()
+        mock_create_store_sink.return_value = mock_store_sink
+        
+        with patch.dict('os.environ', {'STORE_MODE': 'provider', 'DATABASE_URL': 'postgresql://test'}):
+            builder = PipelineBuilder(config=config)
+            
+            spec = PipelineSpec(
+                tenant_id="test_tenant",
+                pipeline_id="test_pipeline",
+                source="synthetic",
+                symbols=["AAPL", "MSFT"],
+                operator="bars",
+                sink="store"
+            )
+            
+            pipeline = builder.build(spec)
+            
+            # Verify components were created
+            mock_source.assert_called_once()
+            mock_operator.assert_called_once()
+            mock_batcher.assert_called_once()
+            mock_create_store_sink.assert_called_once()
+            mock_pipeline.assert_called_once()
+
+    @patch('market_data_pipeline.pipeline_builder.create_store_sink')
+    def test_build_store_sink_amds_not_available_fixed(self, mock_create_store_sink):
+        """Test building store sink when AMDS not available (fixed version)."""
+        config = Mock()
+        config.batch_size = 500
+        config.flush_ms = 100
+        config.sink_queue_max = 100
+        config.pacing_max_per_sec = 1000
+        config.pacing_burst = 1000
+        config.ticks_per_sec = 100
+        config.sink_workers = 2
+        config.drop_policy = "oldest"
+        
+        # Mock the create_store_sink to raise the expected error
+        mock_create_store_sink.side_effect = ConfigurationError("market_data_store AsyncBatchProcessor not installed")
+        
+        with patch('market_data_pipeline.pipeline_builder.AsyncBatchProcessor', None), \
+             patch.dict('os.environ', {'STORE_MODE': 'legacy'}):
             builder = PipelineBuilder(config=config)
             
             spec = PipelineSpec(
